Random

Trait Random 

pub trait Random {
    // Required method
    fn random() -> Self;
}
Trait that manages the object creation for the implemented type with random values.

Required Methods

fn random() -> Self

Returns an object of the implemented type filled with random values.

Dyn Compatibility

This trait is not dyn compatible.

In older versions of Rust, dyn compatibility was called "object safety", so this trait is not object safe.

Implementors